@@ -175,10 +175,10 @@ GNU General Public License for more details.
 #else
 VOID *malloc ();
 VOID *realloc ();
-#endif
 #ifndef getenv
 char *getenv ();
 #endif
+#endif
 
 #if HAVE_LIMITS_H
 #include <limits.h>
@@ -302,3 +302,9 @@ extern int errno;
 #ifndef CVS_CLOSEDIR
 #define CVS_CLOSEDIR closedir
 #endif
+#ifndef CVS_STAT
+#define CVS_STAT stat
+#endif
+#ifndef CVS_FSTAT
+#define CVS_FSTAT fstat
+#endif
