@@ -79,7 +79,8 @@ Checkin (type, finfo, rev, tag, options, message)
 	       changes is if the file contains RCS keywords.  So we if
 	       we are not expanding RCS keywords, we are done.  */
 
-	    if (strcmp (options, "-V4") == 0) /* upgrade to V5 now */
+	    if (options != NULL
+		&& strcmp (options, "-V4") == 0) /* upgrade to V5 now */
 		options[0] = '\0';
 
 	    /* FIXME: If PreservePermissions is on, RCS_cmp_file is
