@@ -162,14 +162,21 @@ import (argc, argv)
      * Could abstract this to valid_module_path, but I don't think we'll need
      * to call it from anywhere else.
      */
-    if ((cp = strstr(argv[0], "CVS")) &&   /* path contains "CVS" AND ... */
-        ((cp == argv[0]) || ISDIRSEP(*(cp-1))) && /* /^CVS/ OR m#/CVS# AND ... */
-        ((*(cp+3) == '\0') || ISDIRSEP(*(cp+3))) /* /CVS$/ OR m#CVS/# */
-       )
+    /* for each "CVS" in path... */
+    cp = argv[0];
+    while ((cp = strstr(cp, "CVS")) != NULL)
     {
-        error (0, 0,
-               "The word `CVS' is reserved by CVS and may not be used");
-        error (1, 0, "as a directory in a path or as a file name.");
+	if ( /* /^CVS/ OR m#/CVS#... */
+	    (cp == argv[0] || ISDIRSEP(*(cp-1)))
+	    /* ...AND /CVS$/ OR m#CVS/# */
+	    && (*(cp+3) == '\0' || ISDIRSEP(*(cp+3)))
+	   )
+	{
+	    error (0, 0,
+		   "The word `CVS' is reserved by CVS and may not be used");
+	    error (1, 0, "as a directory in a path or as a file name.");
+	}
+	cp += 3;
     }
 
     for (i = 1; i < argc; i++)		/* check the tags for validity */
@@ -317,7 +324,8 @@ import (argc, argv)
 
     /* Create the logfile that will be logged upon completion */
     if ((logfp = cvs_temp_file (&tmpfile)) == NULL)
-	error (1, errno, "cannot create temporary file `%s'", tmpfile);
+	error (1, errno, "cannot create temporary file `%s'",
+	       tmpfile ? tmpfile : "(null)");
     /* On systems where we can unlink an open file, do so, so it will go
        away no matter how we exit.  FIXME-maybe: Should be checking for
        errors but I'm not sure which error(s) we get if we are on a system
@@ -1111,6 +1119,7 @@ add_rcs_file (message, rcs, user, add_vhead, key_opt,
     fpuser = NULL;
     if (!preserve_perms || file_type == S_IFREG)
     {
+	errno = 0; /* Standard C doesn't require errno be set on error */
 	fpuser = CVS_FOPEN (userfile,
 			    ((key_opt != NULL && strcmp (key_opt, "b") == 0)
 			     ? "rb"
@@ -1127,6 +1136,7 @@ add_rcs_file (message, rcs, user, add_vhead, key_opt,
 	}
     }
 
+    errno = 0; /* Standard C doesn't require errno be set on error */
     fprcs = CVS_FOPEN (rcs, "w+b");
     if (fprcs == NULL)
     {
@@ -1600,8 +1610,8 @@ import_descend_dir (message, dir, vtag, targc, targv)
     if ( CVS_CHDIR (dir) < 0)
     {
 	ierrno = errno;
-	fperrmsg (logfp, 0, ierrno, "ERROR: cannot chdir to %s", repository);
-	error (0, ierrno, "ERROR: cannot chdir to %s", repository);
+	fperrmsg (logfp, 0, ierrno, "ERROR: cannot chdir to %s", dir);
+	error (0, ierrno, "ERROR: cannot chdir to %s", dir);
 	err = 1;
 	goto out;
     }
