@@ -47,7 +47,7 @@ static int unidiff = 0;
 
 static const char *const patch_usage[] =
 {
-    "Usage: %s %s [-flR] [-c|-u] [-s|-t] [-V %%d]\n",
+    "Usage: %s %s [-flR] [-c|-u] [-s|-t] [-V %%d] [-k kopt]\n",
     "    -r rev|-D date [-r rev2 | -D date2] modules...\n",
     "\t-f\tForce a head revision match if tag/date not found.\n",
     "\t-l\tLocal directory only, not recursive\n",
@@ -56,9 +56,10 @@ static const char *const patch_usage[] =
     "\t-u\tUnidiff format.\n",
     "\t-s\tShort patch - one liner per file.\n",
     "\t-t\tTop two diffs - last change made to the file.\n",
+    "\t-V vers\tUse RCS Version \"vers\" for keyword expansion.\n",
+    "\t-k kopt\tSpecify keyword expansion mode.\n",
     "\t-D date\tDate.\n",
     "\t-r rev\tRevision - symbolic or numeric.\n",
-    "\t-V vers\tUse RCS Version \"vers\" for keyword expansion.\n",
     "(Specify the --help global option for a list of other help options)\n",
     NULL
 };
@@ -86,11 +87,9 @@ patch (argc, argv)
 	{
 	    case 'Q':
 	    case 'q':
-#ifdef SERVER_SUPPORT
 		/* The CVS 1.5 client sends these options (in addition to
 		   Global_option requests), so we must ignore them.  */
 		if (!server_active)
-#endif
 		    error (1, 0,
 			   "-q or -Q must be specified before \"%s\"",
 			   cvs_cmd_name);
@@ -406,6 +405,9 @@ patch_fileproc (callerdat, finfo)
     char *cp1, *cp2;
     FILE *fp;
     int line_length;
+    int dargc = 0;
+    size_t darg_allocated = 0;
+    char **dargv = NULL;
 
     line1 = NULL;
     line1_chars_allocated = 0;
@@ -516,7 +518,8 @@ patch_fileproc (callerdat, finfo)
      */
     if ((fp1 = cvs_temp_file (&tmpfile1)) == NULL)
     {
-	error (0, errno, "cannot create temporary file %s", tmpfile1);
+	error (0, errno, "cannot create temporary file %s",
+	       tmpfile1 ? tmpfile1 : "(null)");
 	ret = 1;
 	goto out;
     }
@@ -525,7 +528,8 @@ patch_fileproc (callerdat, finfo)
 	    error (0, errno, "warning: cannot close %s", tmpfile1);
     if ((fp2 = cvs_temp_file (&tmpfile2)) == NULL)
     {
-	error (0, errno, "cannot create temporary file %s", tmpfile2);
+	error (0, errno, "cannot create temporary file %s",
+	       tmpfile2 ? tmpfile2 : "(null)");
 	ret = 1;
 	goto out;
     }
@@ -534,7 +538,8 @@ patch_fileproc (callerdat, finfo)
 	    error (0, errno, "warning: cannot close %s", tmpfile2);
     if ((fp3 = cvs_temp_file (&tmpfile3)) == NULL)
     {
-	error (0, errno, "cannot create temporary file %s", tmpfile3);
+	error (0, errno, "cannot create temporary file %s",
+	       tmpfile3 ? tmpfile3 : "(null)");
 	ret = 1;
 	goto out;
     }
@@ -559,7 +564,7 @@ patch_fileproc (callerdat, finfo)
 						    (char *) 0, 0)) != -1)
 	    /* I believe this timestamp only affects the dates in our diffs,
 	       and therefore should be on the server, not the client.  */
-	    (void) utime (tmpfile1, &t);
+	    (void) CVS_UTIME (tmpfile1, &t);
     }
     else if (toptwo_diffs)
     {
@@ -582,11 +587,13 @@ patch_fileproc (callerdat, finfo)
 						    (char *)0, 0)) != -1)
 	    /* I believe this timestamp only affects the dates in our diffs,
 	       and therefore should be on the server, not the client.  */
-	    (void)utime (tmpfile2, &t);
+	    (void)CVS_UTIME (tmpfile2, &t);
     }
 
-    switch (diff_exec (tmpfile1, tmpfile2, NULL, NULL, unidiff ? "-u" : "-c",
-                       tmpfile3))
+    if (unidiff) run_add_arg_p (&dargc, &darg_allocated, &dargv, "-u");
+    else run_add_arg_p (&dargc, &darg_allocated, &dargv, "-c");
+    switch (diff_exec (tmpfile1, tmpfile2, NULL, NULL, dargc, dargv,
+		       tmpfile3))
     {
 	case -1:			/* fork/wait failure */
 	    error (1, errno, "fork for diff failed on %s", rcs);
@@ -748,16 +755,33 @@ failed to read diff file header %s for %s: end of file", tmpfile3, rcs);
         free (line1);
     if (line2)
         free (line2);
-    if (CVS_UNLINK (tmpfile1) < 0)
-	error (0, errno, "cannot unlink %s", tmpfile1);
-    if (CVS_UNLINK (tmpfile2) < 0)
-	error (0, errno, "cannot unlink %s", tmpfile2);
-    if (CVS_UNLINK (tmpfile3) < 0)
-	error (0, errno, "cannot unlink %s", tmpfile3);
-    free (tmpfile1);
-    free (tmpfile2);
-    free (tmpfile3);
-    tmpfile1 = tmpfile2 = tmpfile3 = NULL;
+    if (tmpfile1 != NULL)
+    {
+	if (CVS_UNLINK (tmpfile1) < 0)
+	    error (0, errno, "cannot unlink %s", tmpfile1);
+	free (tmpfile1);
+	tmpfile1 = NULL;
+    }
+    if (tmpfile2 != NULL)
+    {
+	if (CVS_UNLINK (tmpfile2) < 0)
+	    error (0, errno, "cannot unlink %s", tmpfile2);
+	free (tmpfile2);
+	tmpfile2 = NULL;
+    }
+    if (tmpfile3 != NULL)
+    {
+	if (CVS_UNLINK (tmpfile3) < 0)
+	    error (0, errno, "cannot unlink %s", tmpfile3);
+	free (tmpfile3);
+	tmpfile3 = NULL;
+    }
+
+    if (dargc)
+    {
+	run_arg_free_p (dargc, dargv);
+	free (dargv);
+    }
 
  out2:
     if (vers_tag != NULL)
