@@ -18,14 +18,13 @@
 const char *const watch_usage[] =
 {
     "Usage: %s %s {on|off|add|remove} [-lR] [-a <action>]... [<path>]...\n",
-    "on/off: turn on/off read-only checkouts of files\n",
-    "add/remove: add or remove notification on actions\n",
-    "-l (on/off/add/remove): Local directory only, not recursive\n",
-    "-R (on/off/add/remove): Process directories recursively (default)\n",
-    "-a (add/remove): Specify what actions, one of\n",
-    "    edit,unedit,commit,all,none (defaults to all, multiple -a\n",
-    "    options are permitted)\n",
-    "(Specify the --help global option for a list of other help options)\n",
+    "on/off: Turn on/off read-only checkouts of files.\n",
+    "add/remove: Add or remove notification on actions.\n",
+    "-l (on/off/add/remove): Local directory only, not recursive.\n",
+    "-R (on/off/add/remove): Process directories recursively (default).\n",
+    "-a (add/remove): Specify what actions, one of: `edit', `unedit',\n",
+    "                 `commit', `all', or `none' (defaults to `all').\n",
+    "(Specify the --help global option for a list of other help options.)\n",
     NULL
 };
 
@@ -407,10 +406,10 @@ watch (argc, argv)
 
 static const char *const watchers_usage[] =
 {
-    "Usage: %s %s [-lR] [files...]\n",
-    "\t-l\tProcess this directory only (not recursive).\n",
-    "\t-R\tProcess directories recursively.\n",
-    "(Specify the --help global option for a list of other help options)\n",
+    "Usage: %s %s [-lR] [<file>]...\n",
+    "-l\tProcess this directory only (not recursive).\n",
+    "-R\tProcess directories recursively (default).\n",
+    "(Specify the --help global option for a list of other help options.)\n",
     NULL
 };
 
