@@ -93,9 +93,7 @@ void wrap_setup()
     else
         wrap_setup_already_done = 1;
 
-#ifdef CLIENT_SUPPORT
     if (!current_parsed_root->isremote)
-#endif
     {
 	char *file;
 
@@ -278,6 +276,7 @@ wrap_add_file (file, temp)
     wrap_kill_temp ();
 
     /* Load the file.  */
+    errno = 0; /* Standard C doesn't require errno be set on error */
     fp = CVS_FOPEN (file, "r");
     if (fp == NULL)
     {
@@ -418,7 +417,8 @@ wrap_add (line, isTemp)
 	switch(opt){
 	case 'f':
 	    /* Before this is reenabled, need to address the problem in
-	       commit.c (see http://www.cvshome.org/docs/infowrapper.html).  */
+	       commit.c (see
+	       http://ximbiot.com/cvs/cvshome/docs/infowrapper.html).  */
 	    error (1, 0,
 		   "-t/-f wrappers not supported by this version of CVS");
 
@@ -432,7 +432,8 @@ wrap_add (line, isTemp)
 	    break;
 	case 't':
 	    /* Before this is reenabled, need to address the problem in
-	       commit.c (see http://www.cvshome.org/docs/infowrapper.html).  */
+	       commit.c (see
+	       http://ximbiot.com/cvs/cvshome/docs/infowrapper.html).  */
 	    error (1, 0,
 		   "-t/-f wrappers not supported by this version of CVS");
 
